@@ -5,7 +5,7 @@ package DBIx::SearchBuilder;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "0.80";
+$VERSION = "0.81";
 
 =head1 NAME
 
@@ -105,18 +105,24 @@ sub _DoSearch {
     my $self = shift;
     my ( $QueryString, $Order );
 
-    $QueryString = "SELECT DISTINCT main.* FROM " . $self->_TableAliases . " ";
+    # The initial SELECT or SELECT DISTINCT is decided later
 
-    $QueryString .= $self->_LeftJoins . " ";
+    $QueryString = $self->_BuildJoins . " ";
 
+    # DISTINCT query only required for multi-table selects
+    if ($QueryString) {
+        $self->_DistinctQuery(\$QueryString, $self->{'table'});
+    } else {
+        $QueryString = "SELECT main.* FROM $QueryString";
+    }
     $QueryString .= $self->_WhereClause . " " . $self->{'table_links'} . " "
       if ( $self->_isLimited > 0 );
 
     # TODO: GroupBy won't work with postgres.
     # $QueryString .= $self->_GroupByClause. " ";
 
-    $QueryString .= $self->_OrderClause;
 
+    $QueryString .= $self->_OrderClause;
 
     $self->_ApplyLimits(\$QueryString);
 
@@ -195,21 +201,24 @@ sub _DoCount {
     #TODO refactor DoSearch and DoCount such that we only have
     # one place where we build most of the querystring
 
-    $QueryString =
-      "SELECT count(DISTINCT main.id) FROM " . $self->_TableAliases . " ";
+    # DISTINCT query only required for multi-table selects
+    if ($#{$self->{'aliases'}} < 0) {
+        $QueryString = "SELECT count(main.id) FROM ";
+    } else {
+        $QueryString = "SELECT count(DISTINCT main.id) FROM ";
+    }
 
-    $QueryString .= $self->_LeftJoins . " ";
+    $QueryString .= $self->_BuildJoins . " ";
 
     $QueryString .= $self->_WhereClause . " " . $self->{'table_links'} . " "
       if ( $self->_isLimited > 0 );
 
 
-    $self->_ApplyLimits(\$QueryString) unless ($all); 
-
-
     print STDERR "DBIx::SearchBuilder->DoSearch Query:  $QueryString\n"
       if ( $self->DEBUG );
 
+
+
     # {{{ get count out of the database
     eval { $self->{'records'} = $self->_Handle->dbh->prepare($QueryString); };
     if ($@) {
@@ -265,6 +274,45 @@ sub _ApplyLimits {
     
 }
 
+# {{{ sub _DistinctQuery
+
+=head2 _DistinctQuery STATEMENTREF
+
+This routine takes a reference to a scalar containing an SQL statement. 
+It massages the statement to ensure a distinct result set is returned.
+
+
+=cut
+
+sub _DistinctQuery {
+    my $self = shift;
+    my $statementref = shift;
+    my $table = shift;
+    $self->_Handle->DistinctQuery($statementref, $table);
+}
+
+# }}}
+
+# {{{ sub _BuildJoins
+
+=head2 _BuildJoins
+
+Build up all of the joins we need to perform this query
+
+=cut
+
+
+sub _BuildJoins {
+    my $self = shift;
+
+    if ($self->_Handle->can('_BuildJoins')) {
+        return ($self->_Handle->_BuildJoins($self));
+    }
+    return( $self->_TableAliases . " ".  $self->_LeftJoins . " ");
+}
+
+# }}}
+
 
 # {{{ sub _LimitClause
 
@@ -1021,32 +1069,13 @@ the alias generated by the join.
 sub Join {
     my $self = shift;
     my %args = ( TYPE   => 'normal',
-                 FIELD1 => undef,
-                 ALIAS1 => undef,
-                 FIELD2 => undef,
-                 TABLE2 => undef,
-                 ALIAS2 => undef,
+                 FIELD1 => undef, ALIAS1 => undef,
+                 TABLE2 => undef, FIELD2 => undef, ALIAS2 => undef,
                  @_ );
 
-    if ( $args{'TYPE'} =~ /LEFT/i ) {
-        my $alias = $self->_GetAlias( $args{'TABLE2'} );
+    $self->_Handle->Join(SearchBuilder => $self, %args);
 
-        $self->{'left_joins'}{"$alias"}{'alias_string'} =
-          " LEFT JOIN $args{'TABLE2'} as $alias ";
 
-        $self->{'left_joins'}{"$alias"}{'criteria'}{'base_criterion'} =
-          " $args{'ALIAS1'}.$args{'FIELD1'} = $alias.$args{'FIELD2'}";
-
-        return ($alias);
-    }
-
-    # we need to build the table of links.
-    my $clause =
-      $args{'ALIAS1'} . "."
-      . $args{'FIELD1'} . " = "
-      . $args{'ALIAS2'} . "."
-      . $args{'FIELD2'};
-    $self->{'table_links'} .= " AND $clause ";
 
 }
 
